use std::{borrow::BorrowMut, collections::HashMap};

use super::Solution;

pub struct Day04 {}

impl Solution for Day04 {
    fn part1(
        &self,
        input: &mut Vec<String>,
    ) -> Result<Box<dyn std::fmt::Display>, Box<dyn std::error::Error>> {
        // Remove Card XXX: at start
        let cards: Vec<&str> = input
            .iter()
            .map(|c| c.split(":").into_iter().last().unwrap())
            .collect();
        let mut ans = 0;

        for card in cards {
            let winning_nums = self.get_count_winning_numbers(card)?;
            if winning_nums > 0 {
                ans += 2u32.pow((winning_nums - 1) as u32);
            }
        }
        Ok(Box::new(ans))
    }

    fn part2(
        &self,
        input: &mut Vec<String>,
    ) -> Result<Box<dyn std::fmt::Display>, Box<dyn std::error::Error>> {
        // Remove Card XXX: at start and append card index
        let mut cards: Vec<(usize, &str)> = input
            .iter()
            .map(|c| c.split(":").into_iter().last().unwrap())
            .into_iter()
            .enumerate()
            .collect();

        for card in cards {}
        // for (i, card) in cards.iter() {
        //     let winning_nums = self.get_count_winning_numbers(card)?;
        //     if winning_nums == 0 {
        //         continue;
        //     }
        //     for c in cards.clone()[(i + 1)..=(i + 1 + winning_nums)].iter() {
        //         cards.push(c.clone());
        //     }
        // }
        // for i in cards.clone() {
        //     println!("{:?}", i);
        // }
        Ok(Box::new(cards.len()))
    }

    fn get_day(&self) -> u8 {
        4
    }
}

impl Day04 {
    fn get_count_winning_numbers(&self, card: &str) -> Result<usize, Box<dyn std::error::Error>> {
        let mut winning_numbers: HashMap<u32, bool> = HashMap::new();
        let mut sep_found = false;
        let mut card_nums = vec![];

        for num in card.split_whitespace() {
            // Are we after the |
            if num == "|" {
                sep_found = true;
                continue;
            }
            let num = num.parse::<u32>()?;

            if sep_found {
                if winning_numbers.contains_key(&num) {
                    card_nums.push(num);
                }
            } else {
                winning_numbers.insert(num, true);
            }
        }
        Ok(card_nums.len())
    }
}

/// Test from puzzle input
#[cfg(test)]
mod test {
    use super::*;
    use crate::*;

    #[test]
    fn part1() {
        let challenge = day04::Day04 {};

        //Complete the Challenge
        let answer = challenge
            .part1(
                utils::get_input(challenge.get_day(), utils::InputType::Test1)
                    .unwrap()
                    .as_mut(),
            )
            .unwrap()
            .to_string();

        assert_eq!(answer, "13");
    }

    #[test]
    fn part2() {
        let challenge = day04::Day04 {};

        //Complete the Challenge
        let answer = challenge
            .part2(
                utils::get_input(challenge.get_day(), utils::InputType::Test2)
                    .unwrap()
                    .as_mut(),
            )
            .unwrap()
            .to_string();

        assert_eq!(answer, "30");
    }
}
